unit frm_Generator;

{*****************************************************************************

 @project PC Web
 @version 1.5

 @author  Cedric TALPAERT <cedric.talpaert@insia.org>
 @since   2004.03.13
 @latest  2006.08.31

 *****************************************************************************}

interface

uses
 jpeg, INIFiles, Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
 Dialogs, StdCtrls, ExtDlgs;

 type
    TParams = record
        Page_Width: Integer;
        Page_Height: Integer;
        MaxPicturesPerPage: Integer;
        Compress: Integer;
        Coef_Pixels: Integer;
        ForceMiniPicturesHorizontal: Boolean;
    end;

function LoadParams(const Path: String): TParams;
function InitListPictures(const Path: String): TStrings;
function ExtractDirName(const Path: String): String;
procedure InitPicturesPerPages(var PicturesPerPage, NbPages: Integer; const NbPictures: Integer);
procedure InitColsAndRows(var NbCols, NbRows: Integer; const PicturesPerPage, Page_Height, Page_Width: Integer);
procedure MakeMiniPictures(const ListPictures: TStrings; const Path: String; Width, Compress, Coef_Pixels: Integer; const Rotate: Boolean);
procedure MakeHTML(const Path: String; const ListPictures: TStrings; const Picture_Width, Width, Height, NbCols, NbRows, Cell_Height: Integer);
procedure ResizeJpeg(srcJpeg, destJpeg: TJpegImage; const MaxSize: integer);
procedure rotateJPEG(Picture: TJPEGImage);

implementation

uses
    frm_Main;

function ExtractDirName(const Path: String): String;
var
    Index: Integer;
begin
  // On recherche le nom du repertoire (sans le chemin !)
  // heu, il doit exister beaucoup plus rapide comme methode...
  Index := Length(Path) - 1;
  While Path[Index] <> '\' do
    Dec(Index);
  // On incremente une fois pour ne pas avoir le '\'
  Inc(Index);
  Result := '';
  While Index <= Length(Path) do
  begin
    Result := Result + Path[Index];
    Inc(Index);
  end;
end;

function LoadParams(const Path: String): TParams;
var
    Inif: TIniFile;
begin
    // Valeurs par dfaut
    Result.Page_Width := 600;
    Result.Page_Height := 900;
    Result.MaxPicturesPerPage := 56;
    Result.Compress := 80;
    Result.Coef_Pixels := 2;

    // On charge les paramatres depuis le fichier ini
    if FileExists(Path) then
    begin
        Inif := TIniFile.Create(Path);
        try
            Result.Page_Width := Inif.ReadInteger('General', 'Width', 0);
            Result.Page_Height := Inif.ReadInteger('General', 'Height', 0);
            Result.MaxPicturesPerPage := Inif.ReadInteger('General', 'MaxPicturesPerPage', 0);
            Result.Compress := Inif.ReadInteger('General', 'Compress', 0);
            Result.Coef_Pixels := Inif.ReadInteger('General', 'Coef_Pixels', 0);
            Result.ForceMiniPicturesHorizontal := Inif.ReadBool('General', 'ForceMiniPicturesHorizontal', True);
        finally
            Inif.Free;
        end;
    end
    else
    begin
        // Creation du fichier ini
        Inif := TIniFile.Create(Path);
        try
            Inif.WriteInteger('General', 'Width', Result.Page_Width);
            Inif.WriteInteger('General', 'Height', Result.Page_Height);
            Inif.WriteInteger('General', 'MaxPicturesPerPage', Result.MaxPicturesPerPage);
            Inif.WriteInteger('General', 'Compress', Result.Compress);
            Inif.WriteInteger('General', 'Coef_Pixels', Result.Coef_Pixels);
        finally
            Inif.Free;
        end;
    end;
end;

function InitListPictures(const Path: String): TStrings;
var
    Index: Integer;
    Sr: TSearchRec;
begin
    // On initialise notre liste de strings
  Result := TStringList.Create;

  // On recherche le premier fichier en .jpg dans le repertoire en cours
  Index := FindFirst(Path + '\*.jpg', faAnyFile, Sr);

    // Pour chaque fichier, on inscrit son nom dans la liste, et on passe au suivant
  While Index = 0 do
  begin
    Result.Add(Sr.Name);
    Index := FindNext(Sr);
  end;
  FindClose(Sr);
end;

procedure InitPicturesPerPages(var PicturesPerPage, NbPages: Integer; const NbPictures: Integer);
begin
  NbPages := NbPictures div PicturesPerPage;
  if (NbPictures mod PicturesPerPage) > 0 then
      Inc(NbPages);
    // l'operation suivante va permetre d'equilibrer le nombre d'images par pages
  PicturesPerPage := NbPictures div NbPages;
  if (NbPictures mod NbPages) > 0 then
    Inc(PicturesPerPage);
end;

procedure InitColsAndRows(var NbCols, NbRows: Integer; const PicturesPerPage, Page_Height, Page_Width: Integer);
var
  coef: integer;
begin
  NbRows := 1;
  NbCols := 1;
  coef := Page_Height div Page_Width;
  if coef = 0 then
    Inc(coef);
  // On incremente d'abord le nombre de lignes, puis le nombre de colones
  while (NbCols * NbRows) < PicturesPerPage do
    if NbCols < (NbRows div coef) then
        Inc(NbCols)
    else
        Inc(NbRows);
end;

procedure MakeMiniPictures(const ListPictures: TStrings; const Path: String; Width, Compress, Coef_Pixels: Integer; const Rotate: Boolean);
var
    Index, NewWidth: integer;
    A, B, C, D, E: Double;
    MyJPEGImage, MyJPEGImage2: TJPEGImage;
begin
  frmMain.Cursor := crHourGlass;
  frmMain.ProgressBar1.Visible := true;
  // Si le repertoire miniPictures n'existe pas, on le cree
  if not DirectoryExists(Path + '\miniPictures') then
    MkDir(Path + '\miniPictures');

  // On redimentionne une copie des images dans le repertoire miniPictures
    MyJPEGImage := TJPEGImage.Create;
    MyJPEGImage2 := TJPEGImage.Create;
    for Index := 0 to ListPictures.Count - 1 do
    try
        frmMain.ProgressBar1.Position := round((Index / ListPictures.Count) * 100);
        MyJPEGImage.LoadFromFile(ListPictures.Strings[Index]);
        // Si on tombe sur une image verticale, on la couche.
        if MyJPEGImage.Height > MyJPEGImage.Width then
        begin
            if Rotate then
                rotateJPEG(MyJPEGImage);
            A := Width;
            B :=  MyJPEGImage.Height / A;
            C := MyJPEGImage.Width / B;
            D := A / C;
            E := C / D;
            NewWidth := round(E);
        end else
            NewWidth := Width;

        ResizeJpeg(MyJPEGImage, MyJPEGImage2, NewWidth * Coef_Pixels);
        MyJPEGImage2.CompressionQuality := Compress;
        MyJPEGImage2.SaveToFile(Path + '\miniPictures\' + ListPictures.Strings[Index]);
    except
        on e:Exception do
            MessageDlg('Une erreur est survenue lors de l''ajout d''une image  la planche contact.'
                + #13 + 'Image provoquant l''erreur: ' + ListPictures.Strings[Index],
                mtWarning, [mbOK], 0);
    end;
    MyJPEGImage.Free;
    MyJPEGImage2.Free;
    frmMain.ProgressBar1.visible := false;
    frmMain.Cursor := crDefault;
end;

procedure MakeHTML(const Path: String; const ListPictures: TStrings; const Picture_Width, Width, Height, NbCols, NbRows, Cell_Height: Integer);
var
    Dochtml, DocCSS: TStrings;
    Index, X, Y: Integer;
begin

  Dochtml := TStringList.Create;
  DocCSS := TStringList.Create;

  try
    // On ecrit le header du document HTML
    Dochtml.Add('<?xml version="1.0" encoding="iso-8859-15"?>');
    Dochtml.Add('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">');
    Dochtml.Add('<html xmlns="http://www.w3.org/1999/xhtml">' + #13);
    Dochtml.Add('<head>');
    Dochtml.add('  <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-15" />');
    Dochtml.Add('  <title> Planche contact - ' + ExtractDirName(Path) + ' </title>');
    Dochtml.Add('  <meta name="GENERATOR" content="PCWeb" />');
    Dochtml.Add('<style type="text/css">');
    // On integre la feuille de style au document HTML
    DocCSS.LoadFromFile(ExtractFileDir(Application.ExeName) + '\' + 'style.css');
    // On y ajoute les dimensions du tableau
    DocCSS.Add(#13 + 'Table {');
    DocCSS.Add('    Width: ' + IntToStr(Width) + 'px;');
    DocCSS.Add('    Height: ' + IntToStr(Height) + 'px;');
    DocCSS.Add('}');
    DocCSS.Add(#13 + 'TD {');
    DocCSS.Add('    Width: ' + IntToStr(Picture_Width) + 'px;');
    DocCSS.Add('    Height: ' + IntToStr(Cell_Height) + 'px;');
    DocCSS.Add('}');
    DocCSS.Add(#13 + 'IMG {');
    DocCSS.Add('    Width: ' + IntToStr(Picture_Width) + 'px;');
    DocCSS.Add('}');
    Dochtml.AddStrings(DocCSS);
    Dochtml.Add('</style>');

    Dochtml.Add('</head>');

    // On ecrit le body du document HTML
    Dochtml.Add('<body>');
    Index := 0;
    while Index < ListPictures.Count do
    begin
      // On cree le tableau qui va contenir les images et leur nom
      Dochtml.Add('<table>');
      Y := 1;
      // Boucle de creation des lignes du tableau HTML
      while (Index < ListPictures.Count) and (Y <= NbRows) do
      begin
          Dochtml.Add('   <tr>');
          X := 1;
          // Boucle de creation des colonnes du tableau HTML
          while (Index < ListPictures.Count) and (X <= NbCols) do
          begin
              // Ecriture du contenu de la cellule
              Dochtml.Add('       <td>');
              if FileExists('./miniPictures/' + ListPictures.Strings[Index]) then
                  Dochtml.Add('           <a href="./'
                    + ListPictures.Strings[Index] + '"><img src="./miniPictures/'
                    + ListPictures.Strings[Index] + '" alt="'
                    + ListPictures.Strings[Index]
                    + ' (reduit)" title=" Voir '
                    + ChangeFileExt(ListPictures.Strings[Index], '')
                    + '" /><br />'
                    + ChangeFileExt(ListPictures.Strings[Index], '') + '</a>')
              else
                  // L'image doit exister, si ca n'est pas le cas on le signale
                  Dochtml.Add('           <a href="./'
                    + ListPictures.Strings[Index]
                    + '"><span class="Error" title="'
                    + ListPictures.Strings[Index]
                    + '"> Aperu non disponible </span><br />'
                    + ChangeFileExt(ListPictures.Strings[Index], '') + '</a>');

              Dochtml.Add('       </td>');
              Inc(X);
              Inc(Index);
          end;
          Dochtml.Add('   </tr>');
          Inc(Y);
        end;
        Dochtml.Add('</table>');
        if Index <> ListPictures.Count then
            Dochtml.Add('<style="page-break-before:always;">');
    end;
    // On ecrit la fin du document HTML
    Dochtml.Add('</body>');
    Dochtml.Add('</html>');

    // On ecrit le fichier sur le disque
    Dochtml.SaveToFile(Path + '\planche_contact.html');

  finally
    DocCSS.Free;
    Dochtml.Free;
  end;
end;

procedure ResizeJpeg(srcJpeg, destJpeg: TJpegImage; const MaxSize: integer);
var
    Bitmap: TBitmap;
    Ratio: double;
begin
    Bitmap := TBitmap.Create;
    try
        Ratio := srcJpeg.Height / srcJpeg.Width;
        Bitmap.Width := MaxSize;
        Bitmap.Height := Round(Bitmap.Width * Ratio);
        Bitmap.Canvas.StretchDraw(Bitmap.Canvas.ClipRect, srcJpeg);
        destJpeg.Assign(Bitmap);
    finally
        Bitmap.Free;
    end;
end;

procedure rotateJPEG(Picture: TJPEGImage);
var
  Bitmap, Bitmap2: TBitmap;
  X, Y: Integer;
begin
  Bitmap := TBitmap.Create;
  Bitmap2 := TBitmap.Create;
  try
      Bitmap.Assign(Picture);
      Bitmap2.Width := Bitmap.Height;
      Bitmap2.Height := Bitmap.Width;
      for X := 0 to Bitmap.Width do
        for Y := 0 to Bitmap.Height do
            Bitmap2.Canvas.Pixels[Bitmap2.Width - Y, X] := Bitmap.Canvas.Pixels[X, Y];
      Picture.Assign(Bitmap2);
  finally
    Bitmap.Free;
    Bitmap2.Free;
  end;
end;

end.
